@@ -51,32 +51,39 @@ info = {
     ],
     "monday": [
         "mon",
-        "monday"
+        "monday",
+        "mo"
     ],
     "tuesday": [
         "tue",
         "tuesday",
+        "tu",
         "Tues"
     ],
     "wednesday": [
         "wed",
-        "wednesday"
+        "wednesday",
+        "we"
     ],
     "thursday": [
         "thu",
-        "thursday"
+        "thursday",
+        "th"
     ],
     "friday": [
         "fri",
-        "friday"
+        "friday",
+        "fr"
     ],
     "saturday": [
         "sat",
-        "saturday"
+        "saturday",
+        "sa"
     ],
     "sunday": [
         "sun",
-        "sunday"
+        "sunday",
+        "su"
     ],
     "am": [
         "am"
@@ -258,6 +265,8 @@ info = {
         "in \\1 week": [
             "in (\\d+[.,]?\\d*) week",
             "in (\\d+[.,]?\\d*) weeks",
+            "in (\\d+[.,]?\\d*) weeks time",
+            "in (\\d+[.,]?\\d*) weeks' time",
             "in (\\d+[.,]?\\d*) wk"
         ],
         "in \\1 year": [
@@ -800,7 +809,7 @@ info = {
             "(?:12\\s+)?midnight": "00:00"
         },
         {
-            "(\\d+[.,]?\\d*)h(\\d+[.,]?\\d*)m?": "\\1:\\2"
+            "(\\d+[.,]?\\d*)h(\\d+[.,]?\\d*)": "\\1:\\2"
         },
         {
             "(?<=from\\s+)now": "in"
