@@ -1,16 +1,15 @@
+from collections import OrderedDict
 from itertools import chain
 
 import regex as re
-from collections import OrderedDict
-
 from dateutil import parser
 
 from dateparser.timezone_parser import pop_tz_offset_from_string, word_is_tz
-from dateparser.utils import normalize_unicode, combine_dicts
+from dateparser.utils import combine_dicts, normalize_unicode
 
-from .dictionary import Dictionary, NormalizedDictionary, ALWAYS_KEEP_TOKENS
+from .dictionary import ALWAYS_KEEP_TOKENS, Dictionary, NormalizedDictionary
 
-NUMERAL_PATTERN = re.compile(r'(\d+)', re.U)
+NUMERAL_PATTERN = re.compile(r"(\d+)", re.U)
 
 
 class Locale:
@@ -42,7 +41,9 @@ class Locale:
 
     def __init__(self, shortname, language_info):
         self.shortname = shortname
-        locale_specific_info = language_info.get("locale_specific", {}).get(shortname, {})
+        locale_specific_info = language_info.get("locale_specific", {}).get(
+            shortname, {}
+        )
         self.info = combine_dicts(language_info, locale_specific_info)
         self.info.pop("locale_specific", None)
 
@@ -83,7 +84,9 @@ class Locale:
         return self._count_words_present_in_the_dictionary(tokens, settings)
 
     def _count_words_present_in_the_dictionary(self, words, settings=None):
-        dictionary = self.clean_dictionary(self._get_split_dictionary(settings=settings))
+        dictionary = self.clean_dictionary(
+            self._get_split_dictionary(settings=settings)
+        )
         dict_cnt = 0
         skip_cnt = 0
         for word in set(words):
@@ -134,67 +137,79 @@ class Locale:
             for pattern, replacement in relative_translations.items():
                 if pattern.match(word):
                     date_string_tokens[i] = pattern.sub(replacement, word)
+                    break
             else:
                 if word in dictionary:
-                    fallback = word if keep_formatting and not word.isalpha() else ''
+                    fallback = word if keep_formatting and not word.isalpha() else ""
                     date_string_tokens[i] = dictionary[word] or fallback
         if "in" in date_string_tokens:
             date_string_tokens = self._clear_future_words(date_string_tokens)
-
-        return self._join(list(filter(bool, date_string_tokens)),
-                          separator="" if keep_formatting else " ", settings=settings)
+        return self._join(
+            list(filter(bool, date_string_tokens)),
+            separator="" if keep_formatting else " ",
+            settings=settings,
+        )
 
     def _translate_numerals(self, date_string):
         date_string_tokens = NUMERAL_PATTERN.split(date_string)
         for i, token in enumerate(date_string_tokens):
             if token.isdecimal():
                 date_string_tokens[i] = str(int(token)).zfill(len(token))
-        return ''.join(date_string_tokens)
+        return "".join(date_string_tokens)
 
     def _get_relative_translations(self, settings=None):
         if settings.NORMALIZE:
             if self._normalized_relative_translations is None:
                 self._normalized_relative_translations = (
-                    self._generate_relative_translations(normalize=True))
+                    self._generate_relative_translations(normalize=True)
+                )
             return self._normalized_relative_translations
         else:
             if self._relative_translations is None:
-                self._relative_translations = self._generate_relative_translations(normalize=False)
+                self._relative_translations = self._generate_relative_translations(
+                    normalize=False
+                )
             return self._relative_translations
 
     def _generate_relative_translations(self, normalize=False):
-        relative_translations = self.info.get('relative-type-regex', {})
+        relative_translations = self.info.get("relative-type-regex", {})
         relative_dictionary = OrderedDict()
         for key, value in relative_translations.items():
             if normalize:
                 value = list(map(normalize_unicode, value))
-            pattern = '|'.join(sorted(value, key=len, reverse=True))
-            pattern = pattern.replace(r'(\d+', r'(?P<n>\d+')
-            pattern = re.compile(r'^(?:{})$'.format(pattern), re.UNICODE | re.IGNORECASE)
+            pattern = "|".join(sorted(value, key=len, reverse=True))
+            pattern = pattern.replace(r"(\d+", r"(?P<n>\d+")
+            pattern = re.compile(
+                r"^(?:{})$".format(pattern), re.UNICODE | re.IGNORECASE
+            )
             relative_dictionary[pattern] = key
         return relative_dictionary
 
     def translate_search(self, search_string, settings=None):
-        dashes = ['-', '——', '—', '～']
+        dashes = ["-", "——", "—", "～"]
         word_joint_unsupported_languages = ["zh", "ja"]
         sentences = self._sentence_split(search_string, settings=settings)
         dictionary = self._get_dictionary(settings=settings)
         translated = []
         original = []
         for sentence in sentences:
-            original_tokens, simplified_tokens = self._simplify_split_align(sentence, settings=settings)
+            original_tokens, simplified_tokens = self._simplify_split_align(
+                sentence, settings=settings
+            )
             translated_chunk = []
             original_chunk = []
             last_token_index = len(simplified_tokens) - 1
             skip_next_token = False
             for i, word in enumerate(simplified_tokens):
                 next_word = simplified_tokens[i + 1] if i < last_token_index else ""
-                current_and_next_joined = self._join_chunk([word, next_word], settings=settings)
+                current_and_next_joined = self._join_chunk(
+                    [word, next_word], settings=settings
+                )
                 if skip_next_token:
                     skip_next_token = False
                     continue
 
-                if word == '' or word == ' ':
+                if word == "" or word == " ":
                     translated_chunk.append(word)
                     original_chunk.append(original_tokens[i])
                 elif (
@@ -204,18 +219,23 @@ class Locale:
                 ):
                     translated_chunk.append(dictionary[current_and_next_joined])
                     original_chunk.append(
-                        self._join_chunk([original_tokens[i], original_tokens[i + 1]], settings=settings)
+                        self._join_chunk(
+                            [original_tokens[i], original_tokens[i + 1]],
+                            settings=settings,
+                        )
                     )
                     skip_next_token = True
                 elif word in dictionary and word not in dashes:
                     translated_chunk.append(dictionary[word])
                     original_chunk.append(original_tokens[i])
-                elif word.strip('()\"\'{}[],.،') in dictionary and word not in dashes:
-                    punct = word[len(word.strip('()\"\'{}[],.،')):]
-                    if punct and dictionary[word.strip('()\"\'{}[],.،')]:
-                        translated_chunk.append(dictionary[word.strip('()\"\'{}[],.،')] + punct)
+                elif word.strip("()\"'{}[],.،") in dictionary and word not in dashes:
+                    punct = word[len(word.strip("()\"'{}[],.،")) :]
+                    if punct and dictionary[word.strip("()\"'{}[],.،")]:
+                        translated_chunk.append(
+                            dictionary[word.strip("()\"'{}[],.،")] + punct
+                        )
                     else:
-                        translated_chunk.append(dictionary[word.strip('()\"\'{}[],.،')])
+                        translated_chunk.append(dictionary[word.strip("()\"'{}[],.،")])
                     original_chunk.append(original_tokens[i])
                 elif self._token_with_digits_is_ok(word):
                     translated_chunk.append(word)
@@ -236,8 +256,12 @@ class Locale:
         for i in range(len(translated)):
             if "in" in translated[i]:
                 translated[i] = self._clear_future_words(translated[i])
-            translated[i] = self._join_chunk(list(filter(bool, translated[i])), settings=settings)
-            original[i] = self._join_chunk(list(filter(bool, original[i])), settings=settings)
+            translated[i] = self._join_chunk(
+                list(filter(bool, translated[i])), settings=settings
+            )
+            original[i] = self._join_chunk(
+                list(filter(bool, original[i])), settings=settings
+            )
         return translated, original
 
     def _get_abbreviations(self, settings):
@@ -245,34 +269,43 @@ class Locale:
         abbreviations = []
         if self._abbreviations is None:
             for item in dictionary:
-                if item.endswith('.') and len(item) > 1:
+                if item.endswith(".") and len(item) > 1:
                     abbreviations.append(item)
             self._abbreviations = abbreviations
         return self._abbreviations
 
     def _sentence_split(self, string, settings):
         abbreviations = self._get_abbreviations(settings=settings)
-        digit_abbreviations = ['[0-9]']  # numeric date with full stop
-        abbreviation_string = ''
+        digit_abbreviations = ["[0-9]"]  # numeric date with full stop
+        abbreviation_string = ""
 
         for abbreviation in abbreviations:
-            abbreviation_string += '(?<! ' + abbreviation[:-1] + ')'  # negative lookbehind
-        if self.shortname in ['fi', 'cs', 'hu', 'de', 'da']:
+            abbreviation_string += (
+                "(?<! " + abbreviation[:-1] + ")"
+            )  # negative lookbehind
+        if self.shortname in ["fi", "cs", "hu", "de", "da"]:
             for digit_abbreviation in digit_abbreviations:
-                abbreviation_string += '(?<!' + digit_abbreviation + ')'  # negative lookbehind
+                abbreviation_string += (
+                    "(?<!" + digit_abbreviation + ")"
+                )  # negative lookbehind
 
-        splitters_dict = {1: r'[\.!?;…\r\n]+(?:\s|$)*',  # most European, Tagalog, Hebrew, Georgian,
-                          # Indonesian, Vietnamese
-                          2: r'[\.!?;…\r\n]+(\s*[¡¿]*|$)|[¡¿]+',  # Spanish
-                          3: r'[|!?;\r\n]+(?:\s|$)+',  # Hindi and Bangla
-                          4: r'[。…‥\.!?？！;\r\n]+(?:\s|$)+',  # Japanese and Chinese
-                          5: r'[\r\n]+',  # Thai
-                          6: r'[\r\n؟!\.…]+(?:\s|$)+'}  # Arabic and Farsi
-        if 'sentence_splitter_group' not in self.info:
+        splitters_dict = {
+            1: r"[\.!?;…\r\n]+(?:\s|$)*",  # most European, Tagalog, Hebrew, Georgian,
+            # Indonesian, Vietnamese
+            2: r"[\.!?;…\r\n]+(\s*[¡¿]*|$)|[¡¿]+",  # Spanish
+            3: r"[|!?;\r\n]+(?:\s|$)+",  # Hindi and Bangla
+            4: r"[。…‥\.!?？！;\r\n]+(?:\s|$)+",  # Japanese and Chinese
+            5: r"[\r\n]+",  # Thai
+            6: r"[\r\n؟!\.…]+(?:\s|$)+",
+        }  # Arabic and Farsi
+        if "sentence_splitter_group" not in self.info:
             split_reg = abbreviation_string + splitters_dict[1]
             sentences = re.split(split_reg, string)
         else:
-            split_reg = abbreviation_string + splitters_dict[self.info['sentence_splitter_group']]
+            split_reg = (
+                abbreviation_string
+                + splitters_dict[self.info["sentence_splitter_group"]]
+            )
             sentences = re.split(split_reg, string)
 
         sentences = filter(None, sentences)
@@ -281,8 +314,10 @@ class Locale:
     def _simplify_split_align(self, original, settings):
         # TODO: Switch to new split method.
         original_tokens = self._word_split(original, settings=settings)
-        simplified_tokens = self._word_split(self._simplify(normalize_unicode(original), settings=settings),
-                                             settings=settings)
+        simplified_tokens = self._word_split(
+            self._simplify(normalize_unicode(original), settings=settings),
+            settings=settings,
+        )
         if len(original_tokens) == len(simplified_tokens):
             return original_tokens, simplified_tokens
 
@@ -297,9 +332,9 @@ class Locale:
                             add_empty = True
                             continue
                         else:
-                            original_tokens.insert(i, '')
+                            original_tokens.insert(i, "")
                 else:
-                    original_tokens.insert(i, '')
+                    original_tokens.insert(i, "")
         else:
             add_empty = False
             for i, token in enumerate(original_tokens):
@@ -311,15 +346,15 @@ class Locale:
                             add_empty = True
                             continue
                         else:
-                            simplified_tokens.insert(i, '')
+                            simplified_tokens.insert(i, "")
                 else:
-                    simplified_tokens.insert(i, '')
+                    simplified_tokens.insert(i, "")
 
         while len(original_tokens) != len(simplified_tokens):
             if len(original_tokens) > len(simplified_tokens):
-                original_tokens.remove('')
+                original_tokens.remove("")
             else:
-                simplified_tokens.remove('')
+                simplified_tokens.remove("")
         return original_tokens, simplified_tokens
 
     def _get_split_dictionary(self, settings):
@@ -332,7 +367,7 @@ class Locale:
     def _split_dict(self, dictionary):
         newdict = {}
         for item in dictionary:
-            if ' ' in item:
+            if " " in item:
                 items = item.split()
                 for i in items:
                     newdict[i] = dictionary[item]
@@ -341,7 +376,7 @@ class Locale:
         return newdict
 
     def _word_split(self, string, settings):
-        if 'no_word_spacing' in self.info:
+        if "no_word_spacing" in self.info:
             return self._split(string, keep_formatting=True, settings=settings)
         else:
             return string.split()
@@ -350,7 +385,10 @@ class Locale:
         tokens = [date_string]
         tokens = list(self._split_tokens_with_regex(tokens, r"(\d+)"))
         tokens = list(
-            self._split_tokens_by_known_words(tokens, keep_formatting, settings=settings))
+            self._split_tokens_by_known_words(
+                tokens, keep_formatting, settings=settings
+            )
+        )
         return tokens
 
     def _split_tokens_with_regex(self, tokens, regex):
@@ -366,20 +404,20 @@ class Locale:
         return list(chain.from_iterable(tokens))
 
     def _join_chunk(self, chunk, settings):
-        if 'no_word_spacing' in self.info:
+        if "no_word_spacing" in self.info:
             return self._join(chunk, separator="", settings=settings)
         else:
-            return re.sub(r'\s{2,}', ' ', " ".join(chunk))
+            return re.sub(r"\s{2,}", " ", " ".join(chunk))
 
     def _token_with_digits_is_ok(self, token):
-        if 'no_word_spacing' in self.info:
-            if re.search(r'[\d\.:\-/]+', token) is not None:
+        if "no_word_spacing" in self.info:
+            if re.search(r"[\d\.:\-/]+", token) is not None:
                 return True
             else:
                 return False
 
         else:
-            if re.search(r'\d+', token) is not None:
+            if re.search(r"\d+", token) is not None:
                 return True
             else:
                 return False
@@ -387,13 +425,41 @@ class Locale:
     def _simplify(self, date_string, settings=None):
         date_string = date_string.lower()
         simplifications = self._get_simplifications(settings=settings)
+
+        if self.info.get("name") == "ru":
+            date_string = self._process_russian_compound_ordinals(
+                date_string, simplifications
+            )
+        else:
+            date_string = self._apply_simplifications(date_string, simplifications)
+
+        return date_string
+
+    def _apply_simplifications(self, date_string, simplifications):
         for simplification in simplifications:
             pattern, replacement = list(simplification.items())[0]
             date_string = pattern.sub(replacement, date_string).lower()
         return date_string
 
+    def _process_russian_compound_ordinals(self, date_string, simplifications):
+        """Process Russian compound ordinals mathematically (двадцать + первое = 21)."""
+        date_string = self._apply_simplifications(date_string, simplifications)
+
+        def replace_number_pairs(match):
+            first_num = int(match.group(1))
+            second_num = int(match.group(2))
+            result = first_num + second_num
+            if 1 <= result <= 31 and first_num in [20, 30] and 1 <= second_num <= 9:
+                return str(result)
+            return match.group(0)
+
+        number_pair_pattern = r"\b(\d+)\s+(\d+)\b"
+        date_string = re.sub(number_pair_pattern, replace_number_pairs, date_string)
+
+        return date_string
+
     def _get_simplifications(self, settings=None):
-        no_word_spacing = eval(self.info.get('no_word_spacing', 'False'))
+        no_word_spacing = eval(self.info.get("no_word_spacing", "False"))
         if settings.NORMALIZE:
             if self._normalized_simplifications is None:
                 self._normalized_simplifications = []
@@ -401,7 +467,7 @@ class Locale:
                 for simplification in simplifications:
                     pattern, replacement = list(simplification.items())[0]
                     if not no_word_spacing:
-                        pattern = r'(?<=\A|\W|_)%s(?=\Z|\W|_)' % pattern
+                        pattern = r"(?<=\A|\W|_)%s(?=\Z|\W|_)" % pattern
                     pattern = re.compile(pattern, flags=re.I | re.U)
                     self._normalized_simplifications.append({pattern: replacement})
             return self._normalized_simplifications
@@ -413,14 +479,14 @@ class Locale:
                 for simplification in simplifications:
                     pattern, replacement = list(simplification.items())[0]
                     if not no_word_spacing:
-                        pattern = r'(?<=\A|\W|_)%s(?=\Z|\W|_)' % pattern
+                        pattern = r"(?<=\A|\W|_)%s(?=\Z|\W|_)" % pattern
                     pattern = re.compile(pattern, flags=re.I | re.U)
                     self._simplifications.append({pattern: replacement})
             return self._simplifications
 
     def _generate_simplifications(self, normalize=False):
         simplifications = []
-        for simplification in self.info.get('simplifications', []):
+        for simplification in self.info.get("simplifications", []):
             c_simplification = {}
             key, value = list(simplification.items())[0]
             if normalize:
@@ -435,7 +501,7 @@ class Locale:
         return simplifications
 
     def _clear_future_words(self, words):
-        freshness_words = {'day', 'week', 'month', 'year', 'hour', 'minute', 'second'}
+        freshness_words = {"day", "week", "month", "year", "hour", "minute", "second"}
         if set(words).isdisjoint(freshness_words):
             words.remove("in")
         return words
@@ -444,7 +510,7 @@ class Locale:
         if not tokens:
             return ""
 
-        capturing_splitters = self._get_splitters(settings)['capturing']
+        capturing_splitters = self._get_splitters(settings)["capturing"]
         joined = tokens[0]
         for i in range(1, len(tokens)):
             left, right = tokens[i - 1], tokens[i]
@@ -479,42 +545,72 @@ class Locale:
     def _set_splitters(self, settings=None):
         splitters = {
             # The ones that split string only if they are not surrounded by letters from both sides:
-            'wordchars': set(),
+            "wordchars": set(),
             # The ones that are not filtered out from tokens after split:
-            'capturing': set(),
+            "capturing": set(),
         }
-        splitters['capturing'] |= set(ALWAYS_KEEP_TOKENS)
+        splitters["capturing"] |= set(ALWAYS_KEEP_TOKENS)
 
         wordchars = self._get_wordchars(settings)
-        skip = set(self.info.get('skip', [])) | splitters['capturing']
+        skip = set(self.info.get("skip", [])) | splitters["capturing"]
         for token in skip:
-            if not re.match(r'^\W+$', token, re.UNICODE):
+            if not re.match(r"^\W+$", token, re.UNICODE):
                 continue
             if token in wordchars:
-                splitters['wordchars'].add(token)
+                splitters["wordchars"].add(token)
 
         self._splitters = splitters
 
     def _set_wordchars(self, settings=None):
         wordchars = set()
         for word in self._get_dictionary(settings):
-            if re.match(r'^[\W\d_]+$', word, re.UNICODE):
+            if re.match(r"^[\W\d_]+$", word, re.UNICODE):
                 continue
             for char in word:
                 wordchars.add(char.lower())
 
-        self._wordchars = wordchars - {" "} | {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"}
+        self._wordchars = wordchars - {" "} | {
+            "0",
+            "1",
+            "2",
+            "3",
+            "4",
+            "5",
+            "6",
+            "7",
+            "8",
+            "9",
+        }
 
     def get_wordchars_for_detection(self, settings):
         if self._wordchars_for_detection is None:
             wordchars = set()
             for word in self._get_dictionary(settings):
-                if re.match(r'^[\W\d_]+$', word, re.UNICODE):
+                if re.match(r"^[\W\d_]+$", word, re.UNICODE):
                     continue
                 for char in word:
                     wordchars.add(char.lower())
-            self._wordchars_for_detection = wordchars - {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
-                                                         ":", "(", ")", "'", "q", "a", "m", "p", " "}
+            self._wordchars_for_detection = wordchars - {
+                "0",
+                "1",
+                "2",
+                "3",
+                "4",
+                "5",
+                "6",
+                "7",
+                "8",
+                "9",
+                ":",
+                "(",
+                ")",
+                "'",
+                "q",
+                "a",
+                "m",
+                "p",
+                " ",
+            }
         return self._wordchars_for_detection
 
     def _generate_dictionary(self, settings=None):
@@ -525,30 +621,32 @@ class Locale:
 
     def to_parserinfo(self, base_cls=parser.parserinfo):
         attributes = {
-            'JUMP': self.info.get('skip', []),
-            'PERTAIN': self.info.get('pertain', []),
-            'WEEKDAYS': [self.info['monday'],
-                         self.info['tuesday'],
-                         self.info['wednesday'],
-                         self.info['thursday'],
-                         self.info['friday'],
-                         self.info['saturday'],
-                         self.info['sunday']],
-            'MONTHS': [self.info['january'],
-                       self.info['february'],
-                       self.info['march'],
-                       self.info['april'],
-                       self.info['may'],
-                       self.info['june'],
-                       self.info['july'],
-                       self.info['august'],
-                       self.info['september'],
-                       self.info['october'],
-                       self.info['november'],
-                       self.info['december']],
-            'HMS': [self.info['hour'],
-                    self.info['minute'],
-                    self.info['second']],
+            "JUMP": self.info.get("skip", []),
+            "PERTAIN": self.info.get("pertain", []),
+            "WEEKDAYS": [
+                self.info["monday"],
+                self.info["tuesday"],
+                self.info["wednesday"],
+                self.info["thursday"],
+                self.info["friday"],
+                self.info["saturday"],
+                self.info["sunday"],
+            ],
+            "MONTHS": [
+                self.info["january"],
+                self.info["february"],
+                self.info["march"],
+                self.info["april"],
+                self.info["may"],
+                self.info["june"],
+                self.info["july"],
+                self.info["august"],
+                self.info["september"],
+                self.info["october"],
+                self.info["november"],
+                self.info["december"],
+            ],
+            "HMS": [self.info["hour"], self.info["minute"], self.info["second"]],
         }
-        name = '{language}ParserInfo'.format(language=self.info['name'])
+        name = "{language}ParserInfo".format(language=self.info["name"])
         return type(name, bases=[base_cls], dict=attributes)
